//
// Created by nicolas on 17/11/23.
//

// You may need to build the project (run Qt uic code generator) to get "ui_MainWindow.h" resolved

#include <QProcess>
#include <QMessageBox>
#include <ui_MainWindow.h>
#include "MainWindow.h"
#include "../interfaces.h"

MainWindow::MainWindow(QMainWindow *parent) :
    QMainWindow(parent), ui(new Ui::MainWindow), items(new QList<PerformanceProfile>) {
    ui->setupUi(this);

    // Initialize porfiles combo box
    initProfilesList();

    // Connect combo box
    //connect(ui->profilesComboBox, QOverload<int>::of(&QComboBox::currentIndexChanged), this, &MainWindow::onSelectionChange);

    // Connect buttons
    connect(ui->confirmBtn, &QPushButton::clicked, this, &MainWindow::onConfirm);
    connect(ui->cancelBtn, &QPushButton::clicked, this, &MainWindow::onCancel);

    // Set window title
    this->setWindowTitle("Nvidia Settings");
}

void MainWindow::initProfilesList() {
    // Generate items
    items->append({ "Eco", 210, 1600});
    items->append({"Low", 210, 1650});
    items->append({ "Normal", 210, 1710 });
    items->append({ "High", 210, 1800 });
    items->append({ "Extreme", 210, 1905 });

    // Add items to combo box
    for (const PerformanceProfile& item : *items) {
        ui->profilesComboBox->addItem(item.name + " \t [" + QString::number(item.maxFrequency) + " MHz]");
    }
    // Set selected item
    ui->profilesComboBox->setCurrentIndex(2);
}

int MainWindow::setPerformanceProfile(const PerformanceProfile *profile) {
    const QString frequencies = QString::number(profile->minFrequency) + "," + QString::number(profile->maxFrequency);
    return QProcess::execute("/usr/bin/sudo", QStringList() << "nvidia-smi" << "-lgc" << frequencies);
}

MainWindow::~MainWindow() {
    delete ui;
    delete items;
}

void MainWindow::onConfirm() {
    const PerformanceProfile profile = items->at(ui->profilesComboBox->currentIndex());
    if (setPerformanceProfile(&profile) != 0) {
        QMessageBox::critical(this, "Error", "Unable to update Nvidia GPU clocks !");
    }
    QApplication::exit();
}

void MainWindow::onCancel() {
    this->close();
}
